#!/usr/bin/env python
# -*- coding: utf-8 -*-


class Estado:
    """
    Clase que representa un estado de un automata finito.
    La clase estado tiene los siguientes atributos:

    -nombre
    -transiciones, que se representan por un mapa de simbolos y
     estados
    -es_final, un booleano que indica si el estado es final
    -alfabeto, el alfabeto total

    La clase estado tiene los siguientes metodos:
    -hallar el hash
    -comprobar la igualdad (se supone que dos objetos con el mismo
     nombre son iguales)
    -anadir letra
    -anadir transicion
    -leer letra
    -escribir el objeto por pantalla
    -devolver el nombre del estado, el alfabeto y si el estado es final
    """

    def __init__(self, nombre, es_final):
        """
        Constructor al que se le pasa el nombre y si es final
        """
        self.nombre = nombre
        self.transiciones = {}
        self.es_final = es_final
        self.alfabeto = set([])

    def __hash__(self):
        """
        Retorna el valor hash de un estado
        """
        return hash(self.nombre)

    def __eq__(self,otro_estado):
        """
        Retorna si dos estados son iguales
        Argumento:
        - `otro_estado`:El estado para comprar si son iguales
        """
        return self.nombre == otro_estado.nombre

    def __str__(self):
        """
        Metodo para hacer la conversion a string
        """
        resultado=''
        resultado += self.nombre + '\n'
        resultado += 'estado final: ' + str(self.es_final) + '\n'
        for l, e in self.transiciones.items():
            resultado += str(l) + ' : '+ e.ver_nombre() + '\n'
        return resultado

    def pon_letra(self, letras):
        """
        Metodo que anade letras al alfabeto
        """
        pass

    def pon_transicion(self, diccionario_nuevas_transiciones):
        """
        Metodo para anadir a nuestras transiciones.
        Si para una letra, hay mas de un estado entonces poner una
        lista de estados como valor del simbolo
        """
        pass

    def lee_letra(self, letra):
        """
        Metodo que devuelve los estados a los que se va despues de
        leer una letra
        """
        pass

    def ver_nombre(self):
        """
        Metodo observador del nombre
        """
        pass

    def ver_alfabeto(self):
        """
        Metodo observador del alfabeto
        """
        pass

    def ver_es_final(self):
        """
        Metodo observador del alfabeto
        """
        pass




class AutomataFinito:
    """
    Clase que representa un automata finito.
    La clase estado tiene dos atributos:

    -estados: el conjunto de estados
    -inicial: el estado inicial
    -alfabeto: el conjunto de letras que es la union de todos los alfabetos de
     los estados
    """

    def __init__(self, estado_inicial ):
        """
        Constructor al que se le pasa el estado inicial
        """
        self.estados=set([])
        self.inicial = estado_inicial
        self.alfabeto = set([])
    def __str__(self):
        """
        Metodo para representar un automata
        """
        resultado = "Inicial :" + str(self.inicial)
        for estado in self.estados:
            resultado += str(estado)
        return resultado

    def poner_alfabeto_total(self):
        """
        Metodo que pone el mismo alfabeto a todos los estados. Es
        conveniente llamarlo despues de anadir un estado
        """
        pass

    def anade_estado(self,otro_estado):
        """
        Metodo que anade un estado al conjunto de estados
        Argumento:
        - `otro_estado`: estado a anadir al conjunto de estados
        """
        pass

    def completar(self):
        """
        Metodo que completa el automata, poniendo un nuevo estado de
        error y anadiendo transiciones a ese nuevo estado para cada
        letra del alfabeto y para cada estado que no tenga definida
        esa letra
        """
        self.error = Estado('Error', False)

    def es_aceptada(self,palabra):
        """
        Comprueba si una palabra es aceptada por el automata
        Argumento:
        - `palabra`: palabra a leer por el automata, la palabra
        deberia ser un objeto iterable (se recomienda una lista o un string)
        """
        pass

    def calculo(self, palabra, estado_inicio):
        """
        Metodo que calcula a que estados se llegan despues de leer una palabra
        Argumentos:
        - `estado_inicio`: Estado desde el que se comienza a leer
        - `palabra`: Palabra que se quiere leer
        """
        pass

    
if __name__ == '__main__':
    #Este metodo main es una forma para testar los programas que se
    #han hecho. Quiza sea conveniente que el alumno modifique los
    #datos de entrada.
    q0 = Estado('q0',True)
    q1 = Estado('q1',False)
    q0.pon_transicion({i:q0 for i in range(4)})
    q1.pon_transicion({i:q0 for i in range(4)})
    q0.pon_transicion({i:q1 for i in range(5,9)})
    q1.pon_transicion({i:q1 for i in range(5,9)})
    A=AutomataFinito(q0)
    A.anade_estado(q1)
    for q in A.calculo([1,2,5],q0):
        print str(q.ver_nombre())
    print str(A.es_aceptada([1,2,5]))
